#!perl
#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#
#   file: t/templates.t #
#
#   Copyright © 2015 Van de Bugger
#
#   This file is part of perl-Dist-Zilla-Plugin-Templates.
#
#   perl-Dist-Zilla-Plugin-Templates is free software: you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free Software Foundation,
#   either version 3 of the License, or (at your option) any later version.
#
#   perl-Dist-Zilla-Plugin-Templates is distributed in the hope that it will be useful, but WITHOUT
#   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with
#   perl-Dist-Zilla-Plugin-Templates. If not, see <http://www.gnu.org/licenses/>.
#
#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

use strict;
use warnings;
use lib 't';

use Test::More;
use Test::Routine::Util;
use Test::Deep qw{ re };

plan tests => 7;

my $role  = 'TemplatesTester';
my $abort = 'Dist::Zilla died in construction: Aborting...';

{
    package MY;
    our $name    = "Assa";
    our $package = "Assa";
    our $version = "0.007";

}

my $files = {
    'README' => [
            '{{$MY::name}} is an example of...',
    ],
    'lib/Assa.pm' => [
        'package {{$MY::package}};',
        '# ABSTRACT: Yoohoo',
        '1;',
    ],
    'lib/Assa/Manual.pod' => [
        '=head1 NAME',
        '',
        '{{$name}} - Err... Aahh...',       # Note: `$name`, not `$MY::name`.
        '/* $version */',
        '',
        '=cut',
    ],
    't/assa.t' => '# This is a part of {{$MY::name}}',
};

run_tests 'InstallModules', $role => {
    files => $files,
    options => {
        templates => ':InstallModules',
    },
    expected => {
        messages => [
        ],
        files => {
            'README' => [
                '{{$MY::name}} is an example of...',
                #^^^^^^^^^^^^^ Not replaced because not an install module.
            ],
            'lib/Assa.pm' => [
                'package Assa;',
                #        ^^^^ Replaced.
                '# ABSTRACT: Yoohoo',
                '1;',
            ],
            'lib/Assa/Manual.pod' => [
                '=head1 NAME',
                '',
                ' - Err... Aahh...',
                #^^^ Replaced with empty string: there is no variable `$name`.
                '/* $version */',
                '',
                '=cut',
            ],
            't/assa.t' => '# This is a part of {{$MY::name}}',
                #                              ^^^^^^^^^^^^^ Not replaced — not an install module.
        },
    },
};

{
    local $MY::name = $MY::name;
    run_tests 'AllFiles', $role => {
        files => $files,
        options => {
            'package' => 'MY',
            'prepend' => '$MY::name = "hohoho";',   # Note: We change the variable value
            'templates' => [
                ':AllFiles',
            ],
        },
        expected => {
            messages => [
            ],
            files => {
                'README' => [
                    'hohoho is an example of...',
                    #^^^^^^ Replaced, not `Assa` because of `prepend` effect.
                ],
                'lib/Assa.pm' => [
                    'package Assa;',
                    #        ^^^^ Replaced.
                    '# ABSTRACT: Yoohoo',
                    '1;',
                ],
                'lib/Assa/Manual.pod' => [
                    '=head1 NAME',
                    '',
                    'hohoho - Err... Aahh...',
                    #^^^^^^ Replaced because of `MY` package context (and `prepend` effect).
                    '/* $version */',
                    '',
                    '=cut',
                ],
            },
        },
    };
}

run_tests 'Non-standard delimiters', $role => {
    files => $files,
    options => {
        'delimiters' => '/* */',
        'package'    => 'MY',
        'templates' => [
            ':AllFiles',
        ],
    },
    expected => {
        messages => [
        ],
        files => {
            'README' => [
                '{{$MY::name}} is an example of...',
                #^^^^^^^^^^^^^ Not replaced.
            ],
            'lib/Assa.pm' => [
                'package {{$MY::package}};',
                #        ^^^^^^^^^^^^^^^^ Not replaced.
                '# ABSTRACT: Yoohoo',
                '1;',
            ],
            'lib/Assa/Manual.pod' => [
                '=head1 NAME',
                '',
                '{{$name}} - Err... Aahh...',
                #^^^^^^^^^ Not replaced.
                '0.007',
                #^^^^^ Replaced.
                '',
                '=cut',
            ],
        },
    }
};

run_tests 'Multiple finders', $role => {
    files => $files,
    options => {
        'templates' => [
            ':InstallModules',      # Multiple file finders work.
            ':TestFiles',
        ],
    },
    expected => {
        files => {
            'lib/Assa.pm'   => [
                'package Assa;',
                #        ^^^^ Replaced because an install module.
                '# ABSTRACT: Yoohoo',
                '1;',
            ],
            't/assa.t'      => '# This is a part of Assa',
            #                                       ^^^^ Replaced because test file.
        },
    },
};

run_tests 'Include', $role => {
    files => {
        'lib/Assa.pm' => 'A {{ include( "include.txt" ); }} B',
        'lib/Foo.pm'  => 'A {{ include( "include.txt" )->fill_in; }} B',
        'lib/Bar.pm'  => 'A {{ include( "include.txt" )->fill_in->indent; }} B',
        #                                              ^^       ^^ Chaining.
        'include.txt' => 'As{{ 2 + 2 }}sa',
    },
    options => { 'templates' => ':InstallModules' },
    expected => {
        files => {
            'lib/Assa.pm' => 'A As{{ 2 + 2 }}sa B',
            #                     ^^^^^^^^^^^ `include` does not expand templates.
            'lib/Foo.pm'  => 'A As4sa B',
            #                    ^^^ but `include()->fill_in` does.
            'lib/Bar.pm'  => 'A     As4sa B',
        },
    },
};

run_tests 'Indent', $role => {
    files => {
        'lib/Assa.pm' => '{{ include( "include.txt" )->indent; }}',
        #                                                   ^^^ Default indent size.
        'lib/Foo.pm'  => '{{ include( "include.txt" )->indent( 2 ); }}',
        #                                                     ^^^ Custom indent size.
        'include.txt' => [
            'line 1',
            '',                         # Empty line should not be indented.
            'line 3',
        ],
    },
    options => { 'templates' => ':InstallModules' },
    expected => {
        files => {
            'lib/Assa.pm' => [
                '    line 1',           # Default indent size is 4 spaces.
                '',                     # Empty line is not indented.
                '    line 3',
            ],
            'lib/Foo.pm' => [
                '  line 1',             # Custom indent size.
                '',                     # Empty line is not indented.
                '  line 3',
            ],
        },
    },
};

run_tests 'Expanded once', $role => {
    files => {
        'lib/Assa.pm' => 'A {{ "{{ 2 + 2 }}" }} B',
        # ^ The file falls to both categories: ":InstallModules" and ":AllFiles".
    },
    options => { 'templates' => [ ':InstallModules', ':AllFiles' ] },
    expected => {
        files => {
            'lib/Assa.pm' => 'A {{ 2 + 2 }} B',
            #                   ^^^^^^^^^^^ Expanded only once.
        },
    },
};

done_testing;

exit( 0 );

# end of file #
